@@ -48,6 +48,7 @@ typedef struct _GVfsJobUnmount         GVfsJobUnmount;
 typedef struct _GVfsJobMountMountable  GVfsJobMountMountable;
 typedef struct _GVfsJobUnmountMountable GVfsJobUnmountMountable;
 typedef struct _GVfsJobOpenForRead     GVfsJobOpenForRead;
+typedef struct _GVfsJobOpenIconForRead     GVfsJobOpenIconForRead;
 typedef struct _GVfsJobSeekRead        GVfsJobSeekRead;
 typedef struct _GVfsJobCloseRead       GVfsJobCloseRead;
 typedef struct _GVfsJobRead            GVfsJobRead;
@@ -138,6 +139,12 @@ struct _GVfsBackendClass
   gboolean (*try_open_for_read) (GVfsBackend *backend,
 				 GVfsJobOpenForRead *job,
 				 const char *filename);
+  void     (*open_icon_for_read) (GVfsBackend *backend,
+                                  GVfsJobOpenIconForRead *job,
+                                  const char *icon_id);
+  gboolean (*try_open_icon_for_read) (GVfsBackend *backend,
+                                      GVfsJobOpenIconForRead *job,
+                                      const char *icon_id);
   void     (*close_read)        (GVfsBackend *backend,
 				 GVfsJobCloseRead *job,
 				 GVfsBackendHandle handle);
@@ -401,7 +408,9 @@ void        g_vfs_backend_set_stable_name                (GVfsBackend        *ba
 void        g_vfs_backend_set_x_content_types            (GVfsBackend        *backend,
 							  char              **x_content_types);
 void        g_vfs_backend_set_icon_name                  (GVfsBackend        *backend,
-							  const char         *icon);
+							  const char         *icon_name);
+void        g_vfs_backend_set_icon                       (GVfsBackend        *backend,
+							  GIcon              *icon);
 void        g_vfs_backend_set_prefered_filename_encoding (GVfsBackend        *backend,
 							  const char         *prefered_filename_encoding);
 void        g_vfs_backend_set_user_visible               (GVfsBackend        *backend,
@@ -418,7 +427,7 @@ const char *g_vfs_backend_get_backend_type               (GVfsBackend        *ba
 const char *g_vfs_backend_get_display_name               (GVfsBackend        *backend);
 const char *g_vfs_backend_get_stable_name                (GVfsBackend        *backend);
 char      **g_vfs_backend_get_x_content_types            (GVfsBackend        *backend);
-const char *g_vfs_backend_get_icon_name                  (GVfsBackend        *backend);
+GIcon      *g_vfs_backend_get_icon                       (GVfsBackend        *backend);
 GMountSpec *g_vfs_backend_get_mount_spec                 (GVfsBackend        *backend);
 GVfsDaemon *g_vfs_backend_get_daemon                     (GVfsBackend        *backend);
 
